//+------------------------------------------------------------------+
//|                                               SignalEconData.mqh |
//|                   Copyright 2009-2017, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals based on Economic Calendar Data r1                 |
//| Type=SignalAdvanced                                              |
//| Name=EconData                                                    |
//| ShortName=EconData                                               |
//| Class=CSignalEconData                                            |
//| Page=signal_econdata                                             |
//| Parameter=MarginFile,string,GBP_cpi-yy.csv,Margin File           |
//| Parameter=ProfitFile,string,USD_core-pce-price-index-yy.csv,Profit File|
//| Parameter=Length,int,6,History Length                            |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSignalEconData.                                           |
//| Purpose: Class of Economic Calendar Data r1.                     |
//|            Derives from class CExpertSignal.                     |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| (Source: https://www.mql5.com/en/articles/2720)
//+------------------------------------------------------------------+
struct SLine
{  string field[];
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSignalEconData   : public CExpertSignal
{
protected:
   string            m_margin_file;
   string            m_profit_file;
   int               m_length;

   SLine             m_profit_data[];
   SLine             m_margin_data[];

public:
                     CSignalEconData(void);
                    ~CSignalEconData(void);

   //--- methods of setting adjustable parameters
   virtual void      MarginFile(string value)
   {  m_margin_file = value;
   }

   virtual void      ProfitFile(string value)
   {  m_profit_file = value;
   }

   virtual void      Length(int value)
   {  m_length = value;
   }

   //--- method of verification of settings
   virtual bool      ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool      InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual int       LongCondition(void);
   virtual int       ShortCondition(void);

protected:
   virtual bool      Load(string DataFile, SLine &Data[]);
   virtual double    Read(datetime Time, SLine &Data[]);
   virtual double    Read(string DB, datetime Time, string Event);
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
void CSignalEconData::CSignalEconData(void) :   m_margin_file("GBP_cpi-yy.csv"),
   m_profit_file("USD_core-pce-price-index-yy.csv"),
   m_length(6)

{
//--- initialization of protected data
   m_used_series = USE_SERIES_OPEN + USE_SERIES_HIGH + USE_SERIES_LOW + USE_SERIES_CLOSE + USE_SERIES_SPREAD + USE_SERIES_TIME;
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
void CSignalEconData::~CSignalEconData(void)
{
}
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CSignalEconData::ValidationSettings(void)
{  if(!CExpertSignal::ValidationSettings())
      return(false);
//--- initial data checks
   if(m_period != PERIOD_MN1)
   {  printf(__FUNCSIG__ + " time frame should be monthly.");
      return(false);
   }
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CSignalEconData::InitIndicators(CIndicators *indicators)
{
//--- check pointer
   if(indicators == NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertSignal::InitIndicators(indicators))
      return(false);
   if(!ReadFileToArrayCSV(m_margin_file, m_margin_data))
      return(false);
   if(!ReadFileToArrayCSV(m_profit_file, m_profit_data))
      return(false);
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| "Voting" that price will grow.                                   |
//+------------------------------------------------------------------+
int CSignalEconData::LongCondition(void)
{  int result = 0;
   m_time.Refresh(-1);
   double _m = Read(m_time.GetData(0), m_margin_data);
   double _p = Read(m_time.GetData(0), m_profit_data);
//double _m = Read("GBP_econ_cal.sqlite",m_time.GetData(0),"cpi-yy");
//double _p = Read("USD_econ_cal.sqlite",m_time.GetData(0),"core-pce-price-index-yy");
//printf(__FUNCSIG__+" m & p CPI is %.2f, & %.2f respectively on: %s.",_m,_p,TimeToString(m_time.GetData(0)));
   if(_m < _p)
   {  result = int(100.0 * ((_p - _m) / _p));
   }
   return(result);
}
//+------------------------------------------------------------------+
//| "Voting" that price will fall.                                   |
//+------------------------------------------------------------------+
int CSignalEconData::ShortCondition(void)
{  int result = 0;
   m_time.Refresh(-1);
   double _m = Read(m_time.GetData(0), m_margin_data);
   double _p = Read(m_time.GetData(0), m_profit_data);
//double _m = Read("GBP_econ_cal.sqlite",m_time.GetData(0),"cpi-yy");
//double _p = Read("USD_econ_cal.sqlite",m_time.GetData(0),"core-pce-price-index-yy");
//printf(__FUNCSIG__+" m & p CPI is %.2f, & %.2f respectively on: %s.",_m,_p,TimeToString(m_time.GetData(0)));
   if(_m > _p)
   {  result = int(100.0 * ((_m - _p) / _m));
   }
   return(result);
}
//+------------------------------------------------------------------+
//| Read Data
//+------------------------------------------------------------------+
double CSignalEconData::Read(datetime Time, SLine &Data[])
{  double _data = 0.0;
   int _rows = ArraySize(Data);
   _data = StringToDouble(Data[0].field[1]);
   for(int i = 0; i < _rows; i++)
   {  if(Time >= StringToTime(Data[i].field[0]))
      {  _data = StringToDouble(Data[i].field[1]);
      }
      else if(Time < StringToTime(Data[i].field[0]))
      {  break;
      }
   }
   return(_data);
}
//+------------------------------------------------------------------+
//| Read Data (Cannot run under strategy tester!)
//+------------------------------------------------------------------+
double CSignalEconData::Read(string DB, datetime Time, string Event)
{  double _data = 0.0;
//--- create or open the database in the common terminal folder
   ResetLastError();
   int _db_handle = DatabaseOpen(DB, DATABASE_OPEN_READWRITE | DATABASE_OPEN_CREATE);
   if(_db_handle == INVALID_HANDLE)
   {  Print("DB: ", DB, " open failed with err: ", GetLastError());
      return(_data);
   }
   string _sql =
      "  SELECT ACTUAL " +
      "  FROM "   +
      "  ( "   +
      "  SELECT ACTUAL "   +
      "  FROM PRICES "  +
      "  WHERE DATE <= '" + TimeToString(Time) + "' "  +
      "  AND EVENT = '" + Event + "'  "  +
      "  ORDER BY DATE DESC   "  +
      "  LIMIT 1  "  +
      "  )  ";
   int _request = DatabasePrepare(_db_handle, _sql);
   if(_request == INVALID_HANDLE)
   {  Print("request failed with err: ", GetLastError());
      DatabaseClose(_db_handle);
      return(_data);
   }
   while(DatabaseRead(_request))
   {  //--- read the values of each field from the obtained entry
      ResetLastError();
      if(!DatabaseColumnDouble(_request, 0, _data))
      {  Print(" DatabaseRead() failed with err: ", GetLastError());
         DatabaseFinalize(_request);
         DatabaseClose(_db_handle);
      }
   }
   return(_data);
}
//+------------------------------------------------------------------+
//| Load Data (Source: https://www.mql5.com/en/articles/2720)
//+------------------------------------------------------------------+
bool ReadFileToArrayCSV(string FileName, SLine & Lines[])
{  ResetLastError();
   int h = FileOpen(FileName, FILE_READ | FILE_ANSI | FILE_CSV | FILE_COMMON, ";");
   if(h == INVALID_HANDLE)
   {  int ErrNum = GetLastError();
      printf("Error opening file %s # %i", FileName, ErrNum);
      return(false);
   }
   int lcnt = 0; // variable for calculating lines
   int fcnt = 0; // variable for calculating line fields
   while(!FileIsEnding(h))
   {  string str = FileReadString(h);
      // new line (new element of the structure array)
      if(lcnt >= ArraySize(Lines)) // structure array completely filled
      {  ArrayResize(Lines, ArraySize(Lines) + 1024); // increase the array size by 1024 elements
      }
      ArrayResize(Lines[lcnt].field, 64); // change the array size in the structure
      Lines[lcnt].field[0] = str; // assign the first field value
      // start reading other fields in the line
      fcnt = 1; // till one element in the line array is occupied
      while(!FileIsLineEnding(h))  // read the rest of fields in the line
      {  str = FileReadString(h);
         if(fcnt >= ArraySize(Lines[lcnt].field)) // field array is completely filled
         {  ArrayResize(Lines[lcnt].field, ArraySize(Lines[lcnt].field) + 64); // increase the array size by 64 elements
         }
         Lines[lcnt].field[fcnt] = str; // assign the value of the next field
         fcnt++; // increase the line counter
      }
      ArrayResize(Lines[lcnt].field, fcnt); // change the size of the field array according to the actual number of fields
      lcnt++; // increase the line counter
   }
   ArrayResize(Lines, lcnt); // change the array of structures (lines) according to the actual number of lines
   FileClose(h);
   return(true);
}
//+------------------------------------------------------------------+
